; ----------------------------------------------------------------------------
; Name:     BIOS MC68000 / MC68020 ***TRAP-15***
; Author:   Steven
; Date:     24-01-2024 		Afgesplits van bios.asm
; Revision: 
; ----------------------------------------------------------------------------

; --- TRAP #15 codes ---------------------------------------------------------
T15ReadStr     equ         2           ;Trap-15, function  2: ReadStr
T15DecOut      equ         3           ;Trap-15, function  3: Decimal out (D1.l)
T15CharIn      equ         5           ;Trap-15, function  5: CharIn
T15CharOut     equ         6           ;Trap-15, function  6: CharOut
T15KeyPressed  equ         7           ;Trap-15, function  7: KeyPressed
T15GetTime     equ         8           ;Trap-15, function  8: GetTime
T15StrOutLf    equ         13          ;Trap-15, function 13: StrOutLf
T15StrOut      equ         14          ;Trap-15, function 14: StrOut
T15CacheCtrl   equ         25          ;Trap-15, function 15: disable/enable cache

;--- TRAP-15 -----------------------------------------------------------------
; 14-11-2021: Easy68K compatible traps. D0.b=taak
;  0 x Display string at (A1), D1.W bytes long (max 255) with Cr&Lf.
;  1 x Display string at (A1), D1.W bytes long (max 255) without CR, LF. (see task 14)
;  2 x Read string from keyboard and store at (A1), NULL terminated, length retuned in D1.W (max 80)
;  3	 Display signed number in D1.L in decimal in smallest field. (see task 15 & 20)
;  4	 Read a number from the keyboard into D1.L.
;  5 x Read single character from the keyboard into D1.B.
;  6 x Display single character in D1.B. 
;  7 x Set D1.B to 1 if keyboard input is pending, otherwise set to 0. Use code 5 to read pending key.
;  8 x Return time in hundredths of a second since midnight in D1.L.
;  9 x Terminate the program. (Halts the simulator)
; 10 x Print the NULL terminated string at (A1) to the default printer. (Not Teesside compatible.)
;      Always send a Form Feed character to end printing. (See below.)
; 11 x Position the cursor at ROW, COL.
;      The high byte of D1.W holds the COL number (0-79),  
;      The low byte holds the ROW number (0-31).
;      0,0 is top left 79,31 is the bottom right. 
;      Out of range coordinates are ignored. 
;      Clear Screen : Set D1.W to $FF00.
; 12	 Keyboard Echo.
;      D1.B = 0 to turn off keyboard echo.
;      D1.B = non zero to enable it (default).
;      Echo is restored on 'Reset' or when a new file is loaded.
; 13 x Display the NULL terminated string at (A1) with CR, LF.
; 14 x Display the NULL terminated string at (A1).
; 15	 Display the unsigned number in D1.L converted to number base (2 through 
;      36) contained in D2.B. For example, to display D1.L in base16 put 16 in D2.B
;      Values of D2.B outside the range 2 to 36 inclusive are ignored.
; 16	 Adjust display properties
;      D1.B = 0 to turn off the display of the input prompt.
;      D1.B = 1 to turn on the display of the input prompt. (default)
;      D1.B = 2 do not display a line feed when Enter pressed during Trap task #2 input 
;      D1.B = 3 display a line feed when Enter key pressed during Trap task #2 input (default) 
;      Other values of D1 reserved for future use. Input prompt display is 
;      enabled by default and by 'Reset' or when a new file is loaded.
; 17	 Display the NULL terminated string at (A1) without CR, LF then
;      Display the decimal number in D1.L.
; 18	 Display the NULL terminated string at (A1) without CR, LF then
;      Read a number from the keyboard into D1.L.
; 19	 Returns current state of up to 4 specified keys or returns key scan code.
;      Pre: D1.L = four 1-byte key codes
;      Post: D1.L contains four 1-byte Booleans. 
;           $FF = corresponding key is pressed, $00 = corresponding key not pressed.
;      Pre: D1.L = $00000000
;      Post: D1.B contains key code of last key pressed
; 20	 Display signed number in D1.L in decimal in field D2.B columns wide.
; 21	 Set Font Color. D1.L = color as $00BBGGRR
;             BB is amount of blue from $00 to $FF
;             GG is amount of green from $00 to $FF
;             RR is amount of red from $00 to $FF
;      D2.B = style by bits,  0 = off, 1 = on
;             bit0 is Bold
;             bit1 is Italic
;             bit2 is Underline
;             bit3 is StrikeOut
; 22	 Read char at Row,Col of text screen.
;      Pre: D1.L = High 16 bits = Row
;                    Low 16 bits = Col
; --- EIGEN TRAPS ------------------------------------------------------------
;      Post: D1.B contains ASCII code of character.
; 23:  LED D1.B=flash(0) / on(1) / off(2)
; 24:  A1 = startadres Fuzix trap-12
; 25:  D1=Disable(0)/Enable(1) cache.


; --- Trp15tbl ---------------------------------------------------------------
; 2024-10-02 No spaces allowed around minus; calculation in vasm will go wrong!
Trp15Tbl:   dc.l        Trp15StrOutLf-Trp15Tbl     ; 0: Display string A1, lengte=D1, met CrLf
            dc.l        Trp15StrOut-Trp15Tbl       ; 1: Display string A1, lengte=D1, zonder CrLf
            dc.l        Trp15ReadStr-Trp15Tbl      ; 2: Read null terminated string (A1), max 64, D1=Lengte
            dc.l        Trp15DecimalOut-Trp15Tbl   ; 3: Display D1.L
            dc.l        Trp15Error-Trp15Tbl        ; 4:
            dc.l        Trp15CharIn-Trp15Tbl       ; 5: Read char from keyboard into D1.B.
            dc.l        Trp15CharOut-Trp15Tbl      ; 6: Display single character in D1.B. 
            dc.l        Trp15CharAvail-Trp15Tbl    ; 7: Pending character: D1.l=1
            dc.l        Trp15TimerTicks-Trp15Tbl   ; 8: Tijd 1/100 seconde in D1.L
            dc.l        Trap0EndProgram-Trp15Tbl   ; 9: Beïndig het programma
            dc.l        Trp15NullStrOut-Trp15Tbl   ;10: Print null terminated string (A1)
            dc.l        Trp15GotoXY-Trp15Tbl       ;11: Cursor to d1(H)=X, d1(L)=Y
            dc.l        Trp15Error-Trp15Tbl        ;12:
            dc.l        Trp15NullStrOutLf-Trp15Tbl ;13: Display null terminated string (A1) with CrLf
            dc.l        Trp15NullStrOut-Trp15Tbl   ;14: Display null terminated string (A1)
            dc.l        Trp15Error-Trp15Tbl        ;15:
            dc.l        Trp15Error-Trp15Tbl        ;16:
            dc.l        Trp15Error-Trp15Tbl        ;17:
            dc.l        Trp15Error-Trp15Tbl        ;18:
            dc.l        Trp15Error-Trp15Tbl        ;19:
            dc.l        Trp15Error-Trp15Tbl        ;20:
            dc.l        Trp15Error-Trp15Tbl        ;21:
            dc.l        Trp15Error-Trp15Tbl        ;22:
            dc.l        Trp15LEDfn-Trp15Tbl        ;23: Custom: LED D1.B=flash(0), on(1), off(2)
            dc.l        Trp15Fuzix-Trp15Tbl        ;24: Custom: A1=TRAP-12 (Fuzix), A2=100 Hz INT
            dc.l        Trp15CacheCtrl-Trp15Tbl    ;25: Custum: D1: disable(0)/enable(1) caches
 

; --- Trap15BIOS -------------------------------------------------------------
;Omdat A0/D0 bewaard worden, mogen deze in de trap-subroutines gebruikt worden!
Trap15BIOS: movem.l     a0/d0,-(SP)       ;Bewaar A0 & D0 op stack
            and.w       #$FF,d0           ;Maak hoge deel nul
            cmp.w       #25,d0            ;Index binnen tabel?
            bhi.s       Trp15End
            asl.w       #2,D0             ;D1.W index voor tabel
            lea         Trp15Tbl(pc),a0   ;Get jump table (reloc!)
            move.l      (a0,d0.w),d0      ;Get jump value
            adda.l      d0,a0             ;Calculate reloc address
            jsr         (a0)              ;Spring naar de functie
Trp15End:   movem.l     (sp)+,a0/d0       ;Herstel A1 & D0
            rte                           ;Terug naar aanroeper

Trp15Error: lea         Trap15Txt(pc),a0  ;Geef melding: not yet implemented
            bsr         StrOut
            asr.w       #2,d0             ;Bereken oorspronkelijk trapnr
            ext.l       d0
            bsr         HexByteOut        ;Print d0 Hex via HexByteOut
            lea         CrLfTxt(pc),a0    ;Print Cr&Lf
            bra         StrOut            ;Print Cr&Lf en return


;--- TRAP#15 STRINGOUT(LF) (D0=0) --------------------------------------------
;14-11-2021: Display string at (A1), D1.W bytes long with Cr&Lf
Trp15StrOutLf0:
            move.b      (a1)+,D0          ;Haal character op
            bsr         CharOut           ;Print deze
Trp15StrOutLf:
            dbf         d1,Trp15StrOutLf0 ;Beginpunt van routine!!!

            lea         CrLfTxt(pc),a0    ;Print Cr&Lf
            bra         StrOut            ;Print Cr&Lf en return


;--- TRAP#15 STRINGOUT (D0=1) ----------------------------------------------
;14-11-2021: Display string at (A1), D1.W bytes long
Trp15StrOut0:
            move.b      (a1)+,D0          ;Haal character op
            bsr         CharOut           ;Print deze
Trp15StrOut:dbf         d1,Trp15StrOut0   ;Beginpunt van routine!!!
            rts                        


;--- TRAP#15 ReadString (D0=2) ----------------------------------------------
;21-11-2021: Return null terminated string at (A1), max 80 bytes; D1.W=length
Trp15ReadStr:
            moveq.l     #80,d0            ;Maximale lengte 80 (=lengte BIOS buffer)
            movea.l     a1,a0             ;Pointer naar buffer in A0
            bsr         ReadStr           ;Lees string (input A0 & D0)
            move.l      d0,d1             ;Return lengte in D1.w
            rts                        


;--- TRAP#15 Display D1 (D0=3) -----------------------------------------------
;01-12-2021: Display D1.L
Trp15DecimalOut:
            move.l      d5,-(sp)          ;Bewaar D5 op stack
            move.l      d1,d5             ;Display D5
            bsr         DecOutD5          ;Display D5.l decimaal
            move.l      (sp)+,d5          ;Herstel D5
            rts                        
            
            
;--- TRAP#15 CHARIN (D0=5) ---------------------------------------------------
;14-11-2021: Display single character in D1.B
Trp15CharIn:bsr         CharIn            ;Lees teken in D0
            move.b      d0,d1             ;Zet character in d0
            rts

            
;--- TRAP#15 CHAROUT (D0=6) --------------------------------------------------
;14-11-2021: Display single character in D1.B
Trp15CharOut:
            move.b      d1,d0             ;Zet character in d0
            bra         CharOut           ;en print deze

            
;--- TRAP#15 CHARAVAILABLE (D0=7) --------------------------------------------
;17-11-2021: Staat er een teken in de buffer? Resultaat in d1=1
Trp15CharAvail:
            tst.w        RecAantal        ;Staat er een teken in de buffer?
            sne          d1
            and.l        #1,d1            ;Resultaat is 0 of 1 (Pascal)
            rts
            

;--- TRAP#15 TIMERTICKS (D0=8) -----------------------------------------------
;17-11-2021: Geeft tijd terug in 1/100 seconde (officieel vanaf 0:00)
Trp15TimerTicks:
            move.l       TimerTicks,d1 ;Zet de tijd in d1.l
            rts

            
;=== TRAP#15 GotoXY (D0=11) ==================================================
;17-11-2021: Cursor to d1(H)=X, d1(L)=Y            
Trp15GotoXY:cmpi.w      #$FF00,d1      ;D1=$FF00 > ClrScr
            beq         ClrScr         ;Wis het scherm met esc,[2J en return

            movem.l     d0/d2,-(sp)    ;Bewaar registers d0 en d2
            move.w      d1,d0          ;d0 wordt rij (0..24)
            and.w       #$00FF,d0      ;Leeg hoge deel van d0
            
            move.w      d1,d2          ;d2 wordt kolom (0..79)
            lsr.w       #8,d2          ;Zet hoge deel in lage deel
            and.w       #$00FF,d2      ;Leeg hoge deel van d2
            bsr         GotoXY         ;Display ANSI-sequence

            movem.l     (sp)+,d0/d2    ;Herstel registers
            rts
 
 ;--- GotoXY ------------------------------------------------------------------
 ;$11 GotoXY:  D0.B = rij[0..24], D2.B = kol[0..79] ***SU-nodig***
 ;1991: Print D0 decimaal met twee cijfers; wordt ook gebruikt door TRAP-15!
 GotoTxt:dc.b     esc,'[',0,';',0,'H',0,0
 GotoXY: movem.l  d1/a0,-(SP)        ;Bewaar registers D1 en A0
         lea      GotoTxt(pc),a0     ;Pointer naar VT200-sequence
         bsr      StrOut             ;print 'esc['

         moveq    #1,d1              ;Zet D0.b+1 in D1.L
         add.b    d0,d1              ;In VT200 regel vanaf 1   
         bsr      DecOutD5           ;print linenr decimaal (D1.w)

         bsr      StrOut             ;print ';'

         moveq    #1,d1              ;Zet D2.b+1 in D1.L
         add.b    d2,d1              ;In Vt200 kolom vanaf 1
         bsr      DecOutD5           ;print colom decimaal

         bsr      StrOut             ;print "H"

         movem.l  (SP)+,d1/A0        ;Herstel D1 en A0
         rts
 
 
 ;--- ClrScr -----------------------------------------------------------------
 ClrScrTxt: dc.b  esc,'[H',esc,'[2J',0
            even
 ClrScr:    move.l   A0,-(SP)
            lea      ClrScrTxt(pc),a0
            bsr      StrOut
            move.l   (SP)+,a0
            rts
 
 
;--- TRAP#15 STRINGOUT(CrLf) (D0=13) -------------------------------------------
;17-11-2021: Display null terminated string at (A1) with Cr&Lf
Trp15NullStrOutLf:
            movea.l     a1,a0          ;Zet stringpointer in A0
            bsr         StrOut         ;Print deze
            lea         CrLfTxt(pc),a0 ;Pointer naar Cr&Lf
            bra         StrOut         ;Print Cr&Lf en return


;--- TRAP#15 STRINGOUT (D0=14) -----------------------------------------------
;17-11-2021: Display null terminated string at (A1)
Trp15NullStrOut:
            movea.l     a1,a0          ;Zet stringpointer in A0
            bra         StrOut         ;Print deze en return


;--- TRAP#15 LEDfunc ---------------------------------------------------------
;LED D1.B=flash(0), on(1), off(2)
Trp15LEDfn: move.b      d1,LEDfunction ;Schrijf LED-functie in geheugen
            bne.s       Trp15LEDfn2    ;Klaar indien D1.b=0 (=knipperfunctie)
            cmpi.b      #1,d1          ;Test op functie 1
            bne.s       Trp15LEDfn1
            bclr.b      #PB2_LED,ViaPB ;D1.b=1: Zet de gele LED aan
            bra.s       Trp15LEDfn2
Trp15LEDfn1:bset.b      #PB2_LED,ViaPB ;D1.b=2: Zet de gele LED uit
Trp15LEDfn2:rts            


;--- TRAP#15 FuzixSysCall ----------------------------------------------------
;Stel Fuzix syscall adres van trap-12 in (A1) en 100Hz INT (A2)
Trp15Fuzix: cmpa.l      #0,a1             ;A1=0? dan NullProc
            bne.s       Trp15Fuzix1
            lea         NullProc(pc),a1   ;Trap-12 wordt NullProc
Trp15Fuzix1:move.l      a1,FuzixSysCall
            cmpa.l      #0,a1             ;A2=0? dan NullProc
            bne.s       Trp15Fuzix2
            lea         NullProc(pc),a2
Trp15Fuzix2:move.l      a2,FuzixSysCall   ;100 Hz int            
            rts
            
            
;--- TRAP#15 Enable/disable cache  -------------------------------------------
;D1: disable(0)/enable(1)/query(-1) caches (68020 en hoger)
Trp15CacheCtrl:            
            if SystemEC020SBC
            cmpi.w      #-1,d1            ;Test op query CACR (-1)
            beq.s       Trp15CacheCtrl1
            
            ori.w       #8,d1             ;Always clear cache
            movec.l     d1,CACR

Trp15CacheCtrl1:                          ;Geef huidige status terug in D1.L
            movec.l     CACR,d1
            else
            clr.l       d1
            endif;
            rts

